package edu.mit.harold.icollections;

import java.util.Collection;




/**
 *
 * A static convenience class for creating efficient immutable bags.
 * <p>
 * This class simply creates MapIBags backed by HashTreeIMaps.
 * 
 * @author harold
 */
public final class HashTreeIBag {
	// not instantiable (or subclassable):
	private HashTreeIBag() {}
	
	private static final MapIBag<Object> EMPTY
		= MapIBag.empty(HashTreeIMap.<Object,Integer>empty());

	/**
	 * @param <E>
	 * @return an empty bag
	 */
	@SuppressWarnings("unchecked")
	public static <E> MapIBag<E> empty() {
		return (MapIBag<E>)EMPTY; }
	
	/**
	 * @param <E>
	 * @param e
	 * @return empty().plus(e)
	 */
	public static <E> MapIBag<E> singleton(final E e) {
		return HashTreeIBag.<E>empty().plus(e); }
	
	/**
	 * @param <E>
	 * @param list
	 * @return empty().plusAll(map)
	 */
	public static <E> MapIBag<E> from(final Collection<? extends E> list) {
		return HashTreeIBag.<E>empty().plusAll(list); }
}
