package edu.mit.harold.icollections;

import java.util.Collection;
import java.util.Map;

/**
 * 
 * An immutable map from non-null keys of type K to non-null values of type V.
 * 
 * @author harold
 *
 * @param <K>
 * @param <V>
 */
public interface IMap<K,V> extends Map<K,V> {
	/**
	 * @param key non-null
	 * @param value non-null
	 * @return a map with the mappings of this but with key mapped to value
	 */
	public IMap<K,V> plus(K key, V value);
	
	/**
	 * @param map
	 * @return this combined with map, with map's mappings used for any keys in both map and this
	 */
	public IMap<K,V> plusAll(Map<? extends K, ? extends V> map);
	
	/**
	 * @param key
	 * @return a map with the mappings of this but with no value for key
	 */
	public IMap<K,V> minus(Object key);
	
	/**
	 * @param keys
	 * @return a map with the mappings of this but with no value for any element of keys
	 */
	public IMap<K,V> minusAll(Collection<?> keys);
}
